// Copyright 1997 Omni Development, Inc.  All rights reserved.
//
// $Header: /LocalDeveloper/Source/CVS/OmniGroup/OmniFoundation/DataStructures.subproj/OFDataBuffer.h,v 1.1 1997/12/19 01:00:56 wjs Exp $

#import "OFObject.h"

#import <Foundation/NSData.h>
#import <Foundation/NSString.h>
#import <Foundation/NSByteOrder.h>
#import <math.h>
#import <architecture/byte_order.h>

#import "NSString-OFExtensions.h"

#define INTERNAL_BUFFER_SIZE 512

typedef struct {
    NSMutableData *data;
    byte tempBuffer[INTERNAL_BUFFER_SIZE];
    int tempBufferCount;
    enum NXByteOrder byteOrder;
} OFDataBuffer;

static inline void
dataBufferInit(OFDataBuffer *dataBuffer)
{
    dataBuffer->data = [[NSMutableData alloc] init];
    dataBuffer->tempBufferCount = 0;
    dataBuffer->byteOrder = NS_UnknownByteOrder;
}

static inline void
dataBufferRelease(OFDataBuffer *dataBuffer)
{
    [dataBuffer->data release];
}

static inline void
dataBufferFlush(OFDataBuffer *dataBuffer)
{
    [dataBuffer->data appendBytes:dataBuffer->tempBuffer length:dataBuffer->tempBufferCount];
    dataBuffer->tempBufferCount = 0;
}

static inline NSData *
dataBufferData(OFDataBuffer *dataBuffer)
{
    dataBufferFlush(dataBuffer);
    return dataBuffer->data;
}

static inline void
dataBufferFlushIfNecessary(OFDataBuffer *dataBuffer)
{
    if (dataBuffer->tempBufferCount < INTERNAL_BUFFER_SIZE)
	return;
    dataBufferFlush(dataBuffer);
}

static inline char
hexCharacterForDigit(int digit)
{
    if (digit < 10)
	return digit + '0';
    else
	return digit + 'a' - 10;
}

static inline void
dataBufferAppendByte(OFDataBuffer *dataBuffer, byte aByte)
{
    dataBufferFlushIfNecessary(dataBuffer);
    dataBuffer->tempBuffer[dataBuffer->tempBufferCount++] = aByte;
}
 
static inline void
dataBufferAppendCharacter(OFDataBuffer *dataBuffer, char ch)
{
    /* Just an alias for backwards compatibility with OmniPDF code */
    dataBufferAppendByte(dataBuffer, ch);
}

static inline void
dataBufferAppendHexForByte(OFDataBuffer *dataBuffer, byte aByte)
{
    dataBufferAppendByte(dataBuffer, hexCharacterForDigit((aByte & 0xf0)>>4));
    dataBufferAppendByte(dataBuffer, hexCharacterForDigit(aByte & 0x0f));
}

static inline void
dataBufferAppendCString(OFDataBuffer *dataBuffer, const char *cString)
{
    const char                 *characterPtr;
    
    for (characterPtr = cString; *characterPtr; characterPtr++)
	dataBufferAppendByte(dataBuffer, *characterPtr);
}
 
static inline void
dataBufferAppendBytes(OFDataBuffer *dataBuffer, const byte *bytes, unsigned int length)
{
    unsigned int                byteCount;
    const byte                 *bytePtr;
    
    for (bytePtr = bytes, byteCount = length; byteCount--; bytePtr++)
	dataBufferAppendByte(dataBuffer, *bytePtr);
}
 

#define SWAP_BYTES(value, swapType)					\
    switch (dataBuffer->byteOrder) {					\
    case NS_UnknownByteOrder:						\
	break;								\
    case NS_LittleEndian:						\
	value = NXSwapHost ## swapType ## ToLittle(value);		\
	break;								\
    case NS_BigEndian:							\
	value = NXSwapHost ## swapType ## ToBig(value);			\
	break;								\
    }

#define APPEND_DATA_OF_TYPE(cType, nameType, swapType)			\
static inline void dataBufferAppend ## nameType				\
	(OFDataBuffer *dataBuffer, cType value)			\
{									\
    SWAP_BYTES(value, swapType);					\
    dataBufferAppendBytes(dataBuffer, (void *)&value, sizeof(cType));	\
}

APPEND_DATA_OF_TYPE(long int, LongInt, Long)
APPEND_DATA_OF_TYPE(short int, ShortInt, Short)
APPEND_DATA_OF_TYPE(unichar, Unichar, Short)

#undef APPEND_DATA_OF_TYPE
#undef SWAP_BYTES

static inline void
dataBufferAppendHexWithReturnsForBytes(OFDataBuffer *dataBuffer,
		      const byte *bytes, unsigned int length)
{
    unsigned int                byteIndex;
    
    byteIndex = 0;
    while (byteIndex < length) {
	dataBufferAppendHexForByte(dataBuffer, bytes[byteIndex]);
	byteIndex++;
	if ((byteIndex % 40) == 0)
	    dataBufferAppendCharacter(dataBuffer, '\n');
    }
}

 
static inline void
dataBufferAppendInteger(OFDataBuffer *dataBuffer, int integer)
{
    int			divisor;
    
    if (integer < 0) {
	integer *= -1;
	dataBufferAppendByte(dataBuffer, '-');
    }
    
    divisor = log10(integer);
    if (divisor < 0)
	divisor = 0;
    divisor = pow(10, divisor);
    while (1) {
	dataBufferAppendByte(dataBuffer, (integer / divisor) + '0');
	if (divisor <= 1)
	    break;
	integer %= divisor;
	divisor /= 10;
    }
}
 
static inline void
dataBufferAppendData(OFDataBuffer *dataBuffer, NSData *data)
{
    dataBufferAppendBytes(dataBuffer, (const byte *)[data bytes], [data length]);
}

static inline void
dataBufferAppendHexWithReturnsForData(OFDataBuffer *dataBuffer, NSData *data)
{
    dataBufferAppendHexWithReturnsForBytes(dataBuffer, (const byte *)[data bytes], [data length]);
}

static inline void
dataBufferAppendString(OFDataBuffer *dataBuffer, NSString *string)
{
    dataBufferAppendCString(dataBuffer, [string cString]);
}

static inline void
dataBufferAppendUnicodeString(OFDataBuffer *dataBuffer, NSString *string)
{
    StringStartLoopThroughCharacters(string, ch) {
        dataBufferAppendUnichar(dataBuffer, ch);
    } StringEndLoopThroughCharacters;
}
